<?php

namespace TDevAgency\Cms\Middleware;

use Closure;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Response;

class AccessMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param Request $request
     * @param Closure $next
     * @return mixed
     */
    public function handle(Request $request, Closure $next)
    {
        if (!$request->user()->is_active) {
            Auth::logout();
            return Redirect::route('cms.login')
                ->withErrors(['email' => 'We don\'t find your credentials or your account is not activated']);
        }

        if ($this->valid($request)) {
            return $next($request);
        }
        return Response::make(null, \Illuminate\Http\Response::HTTP_FORBIDDEN);
    }

    /**
     * @param Request $request
     * @return bool
     */
    private function valid(Request $request): bool
    {
        $permissions = Config::get('cms.role_permissions');
        $routeName = $request->route()->getName();
        $userRoles = $request->user()->roles;

        if ($userRoles[0] instanceof Model) {
            $userRoles = $userRoles->pluck('admin_role')->toArray();
        } else {
            $userRoles = $userRoles->toArray();
        }
        $routeSegments = explode('.', str_replace('cms.', '', $routeName));

        if (empty($routeSegments)) {
            return true;
        }
        foreach ($routeSegments as $i => $segment) {
            if (!isset($permissions[$segment])) {
                return true;
            }
            if (Arr::isAssoc($permissions[$segment])) {
                $permissions = $permissions[$segment];
                continue;
            }

            return !empty(array_intersect($permissions[$segment], $userRoles));
        }

        return true;
    }
}
