<?php

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\HtmlString;

if(! defined('TDEV_AGENCY_CMS_ROOT')){
    define('TDEV_AGENCY_CMS_ROOT', __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR);
}

if(! function_exists('mixCms')){
    /**
     * @param $path
     *
     * @return HtmlString|string
     * @throws Exception
     */
    function mixCms( $path )
    {
        $manifestDirectory = TDEV_AGENCY_CMS_ROOT . 'public';


        static $manifests = [];

        if(! Str::startsWith($path, '/')){
            $path = "/{$path}";
        }

        if($manifestDirectory && ! Str::startsWith($manifestDirectory, '/')){
            $manifestDirectory = "/{$manifestDirectory}";
        }

        if(file_exists(public_path($manifestDirectory . '/hot'))){
            $url = rtrim(file_get_contents(public_path($manifestDirectory . '/hot')));

            if(Str::startsWith($url, ['http://', 'https://'])){
                return new HtmlString(Str::after($url, ':') . $path);
            }

            return new HtmlString("//localhost:8080{$path}");
        }

        $manifestPath = $manifestDirectory . '/mix-manifest.json';

        if(! isset($manifests[$manifestPath])){
            if(! file_exists($manifestPath)){
                throw new Exception('The Mix manifest does not exist.');
            }

            $manifests[$manifestPath] = json_decode(file_get_contents($manifestPath), true);
        }

        $manifest = $manifests[$manifestPath];

        if(! isset($manifest[$path])){
            $exception = new Exception("Unable to locate Mix file: {$path}.");

            if(! app('config')->get('app.debug')){
                report($exception);

                return $path;
            } else{
                throw $exception;
            }
        }

        return new HtmlString(app('config')->get('app.mix_url') . '/cms' . $manifest[$path]);
    }
}

if(! function_exists('inputKeyToName')){
    function inputKeyToName( $expression )
    {
        $keys = explode('.', $expression);
        $name = '';
        if(is_array($keys)){
            array_walk($keys, function ( $item, $key ) use ( &$name ) {
                $name .= $key === 0 ? $item : sprintf('[%s]', $item);
            });
        } else{
            $name = $expression;
        }
        return $name;
    }
}

if(! function_exists('locales_select')){
    function locales_select()
    {
        $locales = [];
        foreach(\Mcamara\LaravelLocalization\Facades\LaravelLocalization::getSupportedLocales() as $key => $value):;
            $locales[$key] = $value['name'];
        endforeach;
        return $locales;
    }
}

if(! function_exists('flag_class')){
    function flag_class( $code )
    {
        $countries = [
            'en' => 'gb',
            'ru' => 'ru',
            'uk' => 'ua',
            'es' => 'es',
            'de' => 'de'
        ];

        return 'flag-icon flag-icon-' . ($countries[$code] ?? null);
    }
}

if(! function_exists('getAdminSideBar')){
    function getAdminSidebar()
    {
        $adminSidebar = Config::get('cms_admin_sidebar');
        ksort($adminSidebar);
        return $adminSidebar;
    }
}

if(! function_exists('getRoles')){
    function getRoles()
    {
        $roles = Config::get('cms.roles');
        array_walk($roles, function ( &$item ) {
            if($item !== null){
                $item = trans($item['name']);
            }
        });
        return $roles;
    }
}


if(! function_exists('hasCmsPermissions')){
    function hasCmsPermissions( $route )
    {
        if(! isset($route['roles'])){
            return true;
        }


        $userRoles = Auth::guard('admin_user')->user()->roles;

        if($userRoles[0] instanceof Model){
            $userRoles = $userRoles->pluck('admin_role')->toArray();
        } else{
            $userRoles = $userRoles->toArray();
        }

        return ! empty(array_intersect($route['roles'], $userRoles));
    }
}
